'From Squeak3.9 of 7 November 2006 [latest update: #7067] on 11 November 2007 at 4:51:53 pm'!
"Change Set:		AsyncFilePluginFixes-dtl
Date:			11 November 2007
Author:			David T. Lewis

These changes fix address calculation problems on 64 bit machines and enable the plugin to work on all combinations of 32bit/64bit image and host. Corresponding changes are required in the external support code, which will affect all platforms (hence a coordinated change of VMMaker and SVN sources is required).

Prerequisites:
VUpdates-1007-dtl, which fixes a pointer arithmetic problem in SmartSyntaxPluginCodeGenerator.

VMMBytesPerWord-dtl, which makes #bytesPerWord and #baseHeaderSize visible to plugins.
"!


!AsynchFilePlugin methodsFor: 'primitives' stamp: 'dtl 10/23/2007 20:58'!
asyncFileValueOf: oop
	"Return a pointer to the first byte of the async file record within the given Smalltalk bytes object, or nil if oop is not an async file record."

	self returnTypeC: 'AsyncFile *'.
	interpreterProxy success:
		((interpreterProxy isIntegerObject: oop) not and:
		 [(interpreterProxy isBytes: oop) and:
		 [(interpreterProxy slotSizeOf: oop) = (self cCode: 'sizeof(AsyncFile)')]]).
	interpreterProxy failed ifTrue: [^ nil].
	^ interpreterProxy firstIndexableField: oop
! !

!AsynchFilePlugin methodsFor: 'primitives' stamp: 'dtl 11/11/2007 11:14'!
primitiveAsyncFileOpen: fileName forWrite: writeFlag semaIndex: semaIndex 
	| fileNameSize fOop f okToOpen |
	self var: #f type: 'AsyncFile *'.
	self primitive: 'primitiveAsyncFileOpen' parameters: #(#String #Boolean #SmallInteger ).
	fileNameSize := interpreterProxy slotSizeOf: (fileName asOop: String).
	"If the security plugin can be loaded, use it to check for permission.
	If not, assume it's ok"
	sCOAFfn ~= 0
		ifTrue: [okToOpen := self cCode: ' ((sqInt (*) (char *, sqInt, sqInt)) sCOAFfn)(fileName, fileNameSize, writeFlag)'.
			okToOpen ifFalse: [^ interpreterProxy primitiveFail]].
	fOop := interpreterProxy instantiateClass: interpreterProxy classByteArray indexableSize: (self cCode: 'sizeof(AsyncFile)').
	f := self asyncFileValueOf: fOop.
	interpreterProxy failed ifFalse: [self cCode: 'asyncFileOpen(f, fileName, fileNameSize, writeFlag, semaIndex)'].
	^ fOop! !

!AsynchFilePlugin methodsFor: 'primitives' stamp: 'dtl 11/11/2007 11:56'!
primitiveAsyncFileReadResult: fhandle intoBuffer: buffer at: start count: num 
	| bufferSize bufferPtr r f count startIndex |
	self var: #f type: 'AsyncFile *'.
	self var: #bufferPtr type: 'char *'.
	self primitive: 'primitiveAsyncFileReadResult' parameters: #(Oop Oop SmallInteger SmallInteger ).

	f := self asyncFileValueOf: fhandle.
	count := num.
	startIndex := start.
	bufferSize := interpreterProxy slotSizeOf: buffer. "in bytes or words"
	(interpreterProxy isWords: buffer)
		ifTrue: ["covert word counts to byte counts"
			count := count * 4.
			startIndex := startIndex - 1 * 4 + 1.
			bufferSize := bufferSize * 4].
	interpreterProxy success: (startIndex >= 1 and: [startIndex + count - 1 <= bufferSize]).
	bufferPtr := (self pointerForOop: buffer) + self baseHeaderSize + startIndex - 1. 	"adjust for zero-origin indexing"
	interpreterProxy failed ifFalse: [r := self cCode: 'asyncFileReadResult(f, bufferPtr, count)'].
	^ r asOop: SmallInteger! !

!AsynchFilePlugin methodsFor: 'primitives' stamp: 'dtl 11/11/2007 11:55'!
primitiveAsyncFileWriteStart: fHandle fPosition: fPosition fromBuffer: buffer at: start count: num 
	| f bufferSize bufferPtr count startIndex |
	self var: #f type: 'AsyncFile *'.
	self var: #bufferPtr type: 'char *'.
	self primitive: 'primitiveAsyncFileWriteStart' parameters: #(Oop SmallInteger Oop SmallInteger SmallInteger ).
	f := self asyncFileValueOf: fHandle.
	interpreterProxy failed ifTrue: [^ nil].

	count := num.
	startIndex := start.
	bufferSize := interpreterProxy slotSizeOf: buffer.	"in bytes or words"
	(interpreterProxy isWords: buffer)
		ifTrue: ["convert word counts to byte counts"
			count := count * self bytesPerWord.
			startIndex := startIndex - 1 * self bytesPerWord + 1.
			bufferSize := bufferSize * self bytesPerWord].
	interpreterProxy success: (startIndex >= 1 and: [startIndex + count - 1 <= bufferSize]).
	bufferPtr := (self pointerForOop: buffer) + self baseHeaderSize + startIndex - 1. 	"adjust for zero-origin indexing"
	interpreterProxy failed ifFalse: [self cCode: 'asyncFileWriteStart(f, fPosition, bufferPtr, count)']! !

